@@ -10,20 +10,50 @@ import numpy as np
 import os
 import shutil
 from ..settings.settings import Settings
+from uquake.core.event import AttribDict
+
+
+def read_srces(fname):
+    with open(fname, 'rb') as srces_file:
+        return pickle.load(srces_file)
+
+
+def read_inventory():
+    if self.files.srces.exists() and use_srces:
+        with open(self.files.srces, 'rb') as srces_file:
+            logger.info('srces will be read from the file and not build'
+                        'from the inventory file')
+            self.srces = pickle.load(srces_file)
+
+        if self.files.inventory.exists():
+            self.inventory = read_inventory(str(self.files.inventory))
+
+    elif self.files.inventory.exists():
+        self.inventory = read_inventory(str(self.files.inventory))
+        self.srces = Srces.from_inventory(self.inventory)
+        logger.info('srces will be build from the inventory file. The '
+                    'srces.pickle file will be replaced.')
+        with open(self.files.srces, 'wb') as srces_file:
+            pickle.dump(self.srces, srces_file)
+
+    elif self.files.srces.exists():
+        logger.info('no inventory file in the project. srces file will be '
+                    'used instead.')
+        with open(self.files.srces, 'rb') as srces_file:
+            self.srces = pickle.load(srces_file)
 
 
 class ProjectManager(object):
 
     inventory_file_name = 'inventory.xml'
 
-    def __init__(self, path, project_name, network_code, use_srces=False,
-                 **kwargs):
+    def __init__(self, base_projects_path, project_name, network_code,
+                 use_srces=False, **kwargs):
         """
-        Interface to manage project providing an interface to selected
-        components of the NonLinLoc software by Anthony Lomax.
+        Interface to manage project and grids.
 
-        :param path: base path
-        :type path: str
+        :param base_projects_path: base project path
+        :type base_projects_path: str
         :param project_name: project name or id
         :type project_name: str
         :param network_code: network name or id
@@ -143,179 +173,151 @@ class ProjectManager(object):
 
         self.project_name = project_name
         self.network_code = network_code
-        self.root_directory = Path(path) / project_name / network_code
-        # create the directory if it does not exist
-        self.root_directory.mkdir(parents=True, exist_ok=True)
 
-        self.inventory_location = self.root_directory / 'inventory'
-        self.inventory_location.mkdir(parents=True, exist_ok=True)
-        self.inventory_file = self.inventory_location / 'inventory.xml'
-        self.srces_file = self.inventory_location / 'srces.pickle'
+        p_vel_base_name = nlloc_grid.VelocityGrid3D.get_base_name(network_code,
+                                                                  'P')
+        s_vel_base_name = nlloc_grid.VelocityGrid3D.get_base_name(network_code,
+                                                                  'S')
+
+        self.base_projects_path = Path(base_projects_path)
+        base_project_path = Path(base_projects_path)
+        root = base_project_path / project_name / network_code
+        self.paths = {'base': base_project_path,
+                      'root': base_project_path / project_name / network_code,
+                      'archives': base_project_path / 'archives',
+                      'inventory': root / 'inventory',
+                      'config': root / 'config',
+                      'velocities': root / 'velocities',
+                      'times': root / 'times'}
+
+        self.paths = AttribDict(self.paths)
+
+        self.files = {'inventory': self.paths.inventory / 'inventory.xml',
+                      'srces': self.paths.inventory / 'srces.pickle',
+                      'settings': self.paths.config / 'settings.toml',
+                      'services_settings': self.paths.config /
+                                           'services_settings',
+                      'p_velocity': self.paths.velocities / p_vel_base_name,
+                      's_velocity': self.paths.velocities / s_vel_base_name}
+
+        self.files = AttribDict(self.files)
+
+        # create the directory if it does not exist
+        self.paths.root.mkdir(parents=True, exist_ok=True)
+        for key in self.paths.keys():
+            path = self.paths[key]
+            logger.info(path)
+            path.mkdir(parents=True, exist_ok=True)
 
         # SETTINGS
 
-        self.config_location = self.root_directory / 'config'
-
-        self.config_location.mkdir(parents=True, exist_ok=True)
-
-        self.settings_file = self.config_location / 'settings.toml'
-        self.service_setting_file = self.config_location / \
-                                    'services_settings.toml'
-
-        if not self.settings_file.is_file():
+        if not self.files.settings.is_file():
             settings_template = Path(os.path.realpath(__file__)).parent / \
                                     '../settings/settings_template.toml'
 
-            shutil.copyfile(settings_template, self.settings_file)
+            shutil.copyfile(settings_template, self.files.settings)
 
-        if not self.service_setting_file.is_file():
+        if not self.files.services_settings.is_file():
             settings_path = Path(os.path.realpath(__file__)).parent.parent
             settings_template = settings_path / \
                                 'settings/services_settings_template.toml'
 
-            shutil.copyfile(settings_template, self.config_location /
-                            'service_settings.toml')
+            shutil.copyfile(settings_template, self.files.services_settings)
 
-        self.settings = Settings(str(self.config_location))
+        self.settings = Settings(str(self.paths.config))
 
         self.srces = None
         self.inventory = None
 
-        if (not self.inventory_file.exists()) and \
-                (not self.srces_file.exists()):
-            logger.warning('the project does not contain an inventory file nor'
-                           'an srces file. to add an inventory file use '
-                           'the add_inventory method. Alternatively, sites '
-                           'information can be provided using an Srces object.'
-                           'Note, however, that an Srces object only contains'
-                           'the sites location information. When '
-                           'the inventory file is present, the Srces object '
-                           'is automatically constructed from the inventory.'
-                           'A Srces object can be added using the add_srces '
-                           'method. If both the Srces and inventory files are'
-                           'present in the project directory, the inventory'
-                           'file takes precedence and the srces file is'
-                           'ignored unless')
-
-        elif self.srces_file.exists() and use_srces:
-            with open(self.srces_file, 'rb') as srces_file:
-                logger.info('srces will be read from the file and not build'
-                            'from the inventory file')
-                self.srces = pickle.load(srces_file)
-
-            if self.inventory_file.exists():
-                self.inventory = read_inventory(str(self.inventory_file))
-
-        elif self.inventory_file.exists():
-            self.inventory = read_inventory(str(self.inventory_file))
+        if self.files.inventory.exists():
+            self.inventory = read_inventory(str(self.files.inventory))
             self.srces = Srces.from_inventory(self.inventory)
-            logger.info('srces will be build from the inventory file. The '
-                        'srces.pickle file will be replaced.')
-            with open(self.srces_file, 'wb') as srces_file:
-                pickle.dump(self.srces, srces_file)
-
-        elif self.srces_file.exists():
-            logger.info('no inventory file in the project. srces file will be '
-                        'used instead.')
-            with open(self.srces_file, 'rb') as srces_file:
+        elif self.files.srces.exists():
+            with open(self.files.srces, 'rb') as srces_file:
                 self.srces = pickle.load(srces_file)
 
-        self.velocity_grid_location = self.root_directory / 'velocities'
-        self.velocity_grid_location.mkdir(parents=True, exist_ok=True)
-
-        p_vel_base_name = nlloc_grid.VelocityGrid3D.get_base_name(network_code,
-                                                                  'P')
-        self.p_velocity_file = self.velocity_grid_location / p_vel_base_name
-        p_files = self.velocity_grid_location.glob(f'{p_vel_base_name}*')
-        if len(list(p_files)) == 0:
-            logger.warning('the project does not contain a p-wave velocity '
-                           'model. to add a p-wave velocity model to the '
-                           'project please use the add_velocity or '
-                           'add_velocities methods.')
-            self.p_velocity = None
-
-        else:
+        self.p_velocity = None
+        try:
             self.p_velocity = nlloc_grid.read_grid(p_vel_base_name,
                                                    path=str(
-                                                 self.velocity_grid_location))
+                                                       self.paths.velocities))
+        except Exception as e:
+            logger.warning(e)
 
-        s_vel_base_name = nlloc_grid.VelocityGrid3D.get_base_name(
-            self.network_code, 'S')
-        self.s_velocity_file = self.velocity_grid_location / s_vel_base_name
-        s_files = self.velocity_grid_location.glob(f'{s_vel_base_name}*')
-        if len(list(s_files)) == 0:
-            logger.warning('the project does not contain a s-wave velocity '
-                           'model. to add a s-wave velocity model to the '
-                           'project please use the add_velocity or '
-                           'add_velocities methods.')
-            self.s_velocity = None
-        else:
+        self.s_velocity = None
+        try:
             self.s_velocity = nlloc_grid.read_grid(s_vel_base_name,
                                                    path=str(
-                                                 self.velocity_grid_location))
+                                                       self.paths.velocities))
+        except Exception as e:
+            logger.warning(e)
 
         self.velocities = None
         if (self.p_velocity is not None) and (self.s_velocity is not None):
             self.velocities = nlloc_grid.VelocityGridEnsemble(self.p_velocity,
                                                               self.s_velocity)
 
-        self.travel_time_grid_location = self.root_directory / 'times'
-        self.travel_time_grid_location.mkdir(parents=True, exist_ok=True)
-        file_list = list(self.travel_time_grid_location.glob('*time*'))
-        if len(file_list) == 0:
-            logger.warning('the project does not contain travel time grids. '
-                           'to initialize the travel-time grid use the '
-                           'init_travel_time_grid method. Note that '
-                           'this require the project to contain both '
-                           'an inventory and a velocities files.')
+        self.paths.times = self.paths.root / 'times'
+        self.paths.times.mkdir(parents=True, exist_ok=True)
+        file_list = list(self.paths.times.glob('*time*'))
 
-            self.travel_times = None
-
-        else:
+        self.travel_times = None
+        try:
             self.travel_times = nlloc_grid.TravelTimeEnsemble.from_files(
-                self.travel_time_grid_location)
+                self.paths.times)
+        except Exception as e:
+            logger.error(e)
+
+    @staticmethod
+    def exists(project_path, project_name, network_code):
+        project_path = Path(project_path) / project_name / network_code
+        return project_path.exists()
 
     def init_travel_time_grid(self):
         """
         initialize the travel time grids
         """
+
+        has_velocity = self.has_p_velocity() or self.has_s_velocity()
+
+        abort = not (has_velocity and self.has_inventory())
+
+        if not self.has_p_velocity():
+            logger.warning('No P-wave velocity found in project')
+
+        if not self.has_s_velocity():
+            logger.warning('No S-wave velocity found in project')
+
+        if not self.has_inventory():
+            logger.warning('No inventory file found in project')
+
+        if abort:
+            logger.warning('travel time grids will not be calculated')
+            return
+
         logger.info('initializing the travel time grids')
         t0 = time()
         seeds = self.srces.locs
         seed_labels = self.srces.labels
 
-        if self.srces is None:
-            return
-
-        if self.p_velocity:
+        if self.has_p_velocity():
             tt_gs_p = self.p_velocity.to_time_multi_threaded(seeds,
                                                              seed_labels)
-
-        if self.s_velocity:
+            self.travel_times = tt_gs_p
+        if self.has_s_velocity():
             tt_gs_s = self.s_velocity.to_time_multi_threaded(seeds,
                                                              seed_labels)
+            self.travel_times = tt_gs_s
 
         if self.p_velocity and self.s_velocity:
             self.travel_times = tt_gs_p + tt_gs_s
 
-        elif self.p_velocity:
-            logger.warning('s-wave velocity model is not set, travel-times '
-                           'will'
-                           'will be generated for ')
-            self.travel_times = tt_gs_p
-
-        elif self.s_velocity:
-            self.travel_times = tt_gs_s
-
-        else:
-            return
-
         # cleaning the directory before writing the new files
 
-        for fle in self.travel_time_grid_location.glob('*time*'):
+        for fle in self.paths.times.glob('*time*'):
             fle.unlink(missing_ok=True)
 
-        self.travel_times.write(self.travel_time_grid_location)
+        self.travel_times.write(self.paths.times)
         t1 = time()
         logger.info(f'done initializing the travel time grids in '
                     f'{t1 - t0:0.2f} seconds')
@@ -333,10 +335,10 @@ class ProjectManager(object):
         :return:
         """
 
-        inventory.write(str(self.inventory_file))
+        inventory.write(str(self.files.inventory))
         self.srces = Srces.from_inventory(inventory)
         if create_srces_file:
-            with open(self.srces_file, 'wb') as srces_file:
+            with open(self.files.srces, 'wb') as srces_file:
                 pickle.dump(self.srces, srces_file)
 
         if initialize_travel_time:
@@ -377,7 +379,7 @@ class ProjectManager(object):
                                'will be added.')
 
         self.srces = srces
-        with open(self.srces_file, 'wb') as srces_file:
+        with open(self.files.srces, 'wb') as srces_file:
             pickle.dump(self.srces, srces_file)
 
         if initialize_travel_time:
@@ -404,28 +406,37 @@ class ProjectManager(object):
         travel time grids to be incompatible.
         """
 
-        # velocities.write(path=self.velocity_grid_location)
+        # velocities.write(path=self.paths.velocities)
 
         self.velocities = velocities
 
         for key in velocities.keys():
-            self.add_velocity(velocities[key],
-                              initialize_travel_time=False)
+            self.add_velocity(velocities[key])
 
-        if initialize_travel_time:
-            self.init_travel_time_grid()
-        else:
-            logger.warning('the travel time grids will not be initialized, '
-                           'the inventory and the travel time grids might '
-                           'be out of sync. To initialize the travel time '
-                           'grids make sure initialize_travel_time is set to '
-                           'True')
+        self.init_travel_time_grid()
+
+    def velocity_versioning_handler(self, phase):
+        """
+        returns the version number of the to be archived velocity model
+        :return: int
+        """
+        file_list = [fle for fle in self.paths.archives.glob(f'*.{phase}.*')]
+        # file_list = os.listdir(self.paths.archives / )
+        if not file_list:
+            return 1
+
+        exts = []
+        for element in file_list:
+            ext = int(element.split('_')[-1])
+            exts.append(ext)
+
+        return np.max(exts) + 1
 
     def add_velocity(self, velocity, initialize_travel_time=True):
         """
         add P- or S-wave velocity model to the project
         :param velocity: p-wave velocity model
-        :type velocity: nlloc.grid.VelocityGrid3D
+        :type velocity: uquake.grid.nlloc.VelocityGrid3D
         :param initialize_travel_time: if true initialize the travel time grids
 
         ..warning:: travel time should be initialized when the sites/srces
@@ -433,20 +444,100 @@ class ProjectManager(object):
         travel time grids to be incompatible.
         """
 
-        velocity.write(path=self.velocity_grid_location)
-
         if velocity.phase.upper() == 'P':
             self.p_velocity = velocity
-
-        else:
+        elif velocity.phase.upper() == 'S':
             self.s_velocity = velocity
 
-        if initialize_travel_time:
+        velocity.write(self.paths.velocities)
+
+        try:
             self.init_travel_time_grid()
-        else:
-            logger.warning('the inventory and the travel time grids might'
-                           'be out of sync.')
+        except Exception as e:
+            logger.warning(e)
+
+    def list_active_projects(self):
+        """
+        List all active projects
+        :return: list of active projects
+        """
+        projects = []
+        for project in self.base_projects_path.glob('*'):
+            if project.name == 'archives':
+                continue
+            if project.is_dir():
+                projects.append(project.name)
+
+        return projects
+
+    def list_active_networks_project(self, project=None):
+        """
+        List all active networks
+        :param project: the project for which to list the networks
+        :return: a list of networks
+        """
+        if project is None:
+            project = self.project_name
+
+        project_directory = self.base_projects_path / project
+
+        if not project_directory.exists():
+            logger.warning(f'the project {project} does not exists')
+            return
+
+        networks = []
+        for network in project_directory.glob('*'):
+            if not network.is_dir():
+                continue
+            networks.append(network.name)
+
+        return networks
+
+    def list_all_active_networks(self):
+        """
+        list all active projects and networks
+        :return: a dictionary of list
+        """
+
+        out_dict = {}
+        for project in self.list_active_projects():
+            networks = self.list_active_networks_project(project=project)
+            out_dict[project] = networks
+
+        return out_dict
+
+    def list_all_archived_networks(self):
+        """
+        List all archived projects and networks
+        :return: dictionary of list where the keys are the projects name
+        """
+
+        out_dict = {}
+        for project in self.paths.archives.glob('*'):
+            if not project.is_dir():
+                continue
+            archived_project_dir = self.paths.archives / project.name
+            out_dict[project.name] = []
+            for network in archived_project_dir.glob('*'):
+                if not network.is_dir():
+                    continue
+                out_dict[project.name].append(network.name)
+
+        return out_dict
+
+    def has_p_velocity(self):
+        if self.p_velocity:
+            return True
+        return False
+
+    def has_s_velocity(self):
+        if self.s_velocity:
+            return True
+        return False
+
+    def has_inventory(self):
+        if self.inventory or self.srces:
+            return True
+        return False
 
-    def clean_project(self):
-        pass
 
